/*
 * The most basic *cough* possible example of a MoDapple slot module.
 * Based on the behavior observed when probing the Agat emulator.
 */

/* for integer types */
#include <stdint.h>

static uint8_t intbasic[12288];
static int enable;

/*
 * Module information
 */
__declspec(dllexport) uint32_t mod_type=0x544F4C53;
__declspec(dllexport) uint8_t *mod_name="Integer Basic Firmware Card";
__declspec(dllexport) uint8_t *mod_pref="intbasic";
__declspec(dllexport) uint8_t *mod_vers="0.01";
__declspec(dllexport) uint8_t *mod_copr="Copyright 2017 Steve Nickolas\n";

/*
 * Tell the shell to load our ROM for us
 */
__declspec(dllexport) uint8_t  *intbasic_preload_name="intbasic.rom";
__declspec(dllexport) uint16_t intbasic_preload_size=12288;
__declspec(dllexport) uint8_t  *intbasic_preload_ptr=intbasic;

/*
 * The firmware card is activated when an EVEN address in slot i/o space is
 * WRITTEN to, and deactivated when an ODD address is WRITTEN to.
 */
__declspec(dllexport) void intbasic_write16 (uint16_t a, uint8_t v)
{
 enable=1-(a&1);
}

/*
 * We are a slot 0 card; we don't have a rom space.
 */
__declspec(dllexport) void intbasic_write256 (uint16_t a, uint8_t v)
{
}

/*
 * We don't have a shared memory space.
 */
__declspec(dllexport) void intbasic_write2048 (uint16_t a, uint8_t v)
{
}

/*
 * Reading I/O space has no effect.
 */
__declspec(dllexport) uint8_t intbasic_read16 (uint16_t a)
{
 return 255;
}

/*
 * We are a slot 0 card; we don't have a rom space.
 */
__declspec(dllexport) uint8_t intbasic_read256 (uint16_t a)
{
 return 255;
}

/*
 * We don't have a shared memory space.
 */
__declspec(dllexport) uint8_t intbasic_read2048 (uint16_t a)
{
 return 255;
}

/*
 * No change to what writing to any memory address will do.
 */
__declspec(dllexport) int16_t intbasic_override_wr (uint16_t a, uint8_t v)
{
 return -1;
}

/*
 * Intercept READS to the top 12K of memory, IF enabled.
 * Otherwise do nothing.
 */
__declspec(dllexport) int16_t intbasic_override_rd (uint16_t a)
{
 if (a>=0xD000)
 {
  if (!enable) return -1;
  return intbasic[a-0xD000];
 }
 return -1;
}

/*
 * Disable ourself when Reset is hit.
 */
__declspec(dllexport) int intbasic_reset (void)
{
 enable=0;
 return 0;
}

/*
 * Start out disabled.
 */
__declspec(dllexport) int intbasic_init (void)
{
 enable=0;
 return 0;
}

/*
 * Nothing to do when deinitializing.
 */
__declspec(dllexport) int intbasic_deinit (void)
{
 return 0;
}
